extends Node2D

var orig_mat := ShaderMaterial.new()
var orig_mat2 := ShaderMaterial.new()
var orig_mat3 := ShaderMaterial.new()

func _ready():
	print("READY");

	Engine.max_fps = 60

	var ie_space = InputEventKey.new()
	ie_space.physical_keycode = KEY_SPACE

	var ie_enter = InputEventKey.new()
	ie_enter.physical_keycode = KEY_ENTER

	InputMap.action_erase_event("interact", ie_enter)
	InputMap.action_add_event("interact", ie_space)
	
	orig_mat = $Sprite2D.material
	orig_mat2 = $SpriteIcon.material
	orig_mat3 = $Polygon2D.material


func _physics_process(delta):
	if Input.is_action_just_pressed("interact"):
		print("interact")


# https://docs.godotengine.org/en/stable/tutorials/shaders/compute_shaders.html
func compute():
	# Create a local rendering device.
	var rd := RenderingServer.create_local_rendering_device()
	if not rd:
		print("rd is NULL")
		return
	# Load GLSL shader
	var shader_file: RDShaderFile = load("res://compute_example.glsl")
	var shader_spirv: RDShaderSPIRV = shader_file.get_spirv()

	var shader := rd.shader_create_from_spirv(shader_spirv)

	# Prepare our data. We use floats in the shader, so we need 32 bit.
	var input := PackedFloat32Array([1, 2, 3, 4, 5, 6, 7, 8, 9, 10])
	var input_bytes := input.to_byte_array()

	# Create a storage buffer that can hold our float values.
	# Each float has 4 bytes (32 bit) so 10 x 4 = 40 bytes
	var buffer := rd.storage_buffer_create(input_bytes.size(), input_bytes)

	# Create a uniform to assign the buffer to the rendering device
	var uniform := RDUniform.new()
	uniform.uniform_type = RenderingDevice.UNIFORM_TYPE_STORAGE_BUFFER
	uniform.binding = 0 # this needs to match the "binding" in our shader file
	uniform.add_id(buffer)
	# the last parameter (the 0) needs to match the "set" in our shader file
	var uniform_set := rd.uniform_set_create([uniform], shader, 0)

	# Create a compute pipeline
	var pipeline := rd.compute_pipeline_create(shader)
	var compute_list := rd.compute_list_begin()
	rd.compute_list_bind_compute_pipeline(compute_list, pipeline)
	rd.compute_list_bind_uniform_set(compute_list, uniform_set, 0)
	rd.compute_list_dispatch(compute_list, 5, 1, 1)
	rd.compute_list_end()

	# Submit to GPU and wait for sync
	rd.submit()
	rd.sync()

	# Read back the data from the buffer
	var output_bytes := rd.buffer_get_data(buffer)
	var output := output_bytes.to_float32_array()
	print("Input: ", input)
	print("Output: ", output)
	$Output.text = str(output)


func _on_button_compute_pressed():
	compute()


func _on_button_add_shader_pressed():
	var mat := ShaderMaterial.new()
	var mat2 := ShaderMaterial.new()
	var mat3 := ShaderMaterial.new()

	var start_time = Time.get_ticks_msec()
	mat.shader = load("res://global_color_canvas_item.gdshader")
	mat2.shader = mat.shader
	mat3.shader = mat.shader

	print("set_shader_parameter")
	#mat.set_shader_parameter("global_color", Vector4(1.0, 0.5, 0.5, 0.5))
	#mat2.set_shader_parameter("global_color", Vector4(1.0, 1.0, 0.5, 0.5))
	#mat3.set_shader_parameter("global_color", Vector4(0.0, 1.0, 1.0, 0.5))
	
	var time = Time.get_ticks_msec() - start_time
	print("Shader loaded in %s ms" % time)

	$Sprite2D.material = mat
	$SpriteIcon.material = mat2
	$Polygon2D.material = mat3
	
	pass



func _on_timer_timeout():
	print("_on_timer_timeout")
	_on_button_add_shader_pressed()
	pass


func _on_timer_navigate_to_3d_timeout():
	#print("_on_timer_navigate_to_3d_timeout")
	#var scene_3d = preload("res://test_3d.tscn").instantiate()
	#get_tree().root.get_node("Node2D").hide()
	#get_tree().root.add_child(scene_3d)
	pass


func _on_timer_reset_timeout() -> void:
	print("RESET")
	var temp_shader := load("res://temp_uniform.gdshader")
	var mat := ShaderMaterial.new()
	mat.shader = temp_shader
	
	var static_red_shader := load("res://static_red.gdshader")
	var mat_static_red := ShaderMaterial.new()
	mat_static_red.shader = static_red_shader
	
	var color_array_shader := load("res://color_array.gdshader")
	var mat_array := ShaderMaterial.new()
	mat_array.shader = color_array_shader
	
	
	
	var color_array = []
	color_array.append(Vector4(0.5, 0.5, 0.1, 1.0))
	color_array.append(Vector4(0.5, 0.1, 0.5, 1.0))
	color_array.append(Vector4(0.1, 0.5, 0.5, 1.0))
	

	mat_array.set_shader_parameter("color_array", color_array)
	mat_array.set_shader_parameter("color_index", 2)
	mat_array.set_shader_parameter("pos_offset", Vector2(50.0, 50.0))
	
	$Sprite2D.material = orig_mat
	$SpriteIcon.material = mat_array #orig_mat2
	$Polygon2D.material = mat_static_red #orig_mat3
